import os, struct, select
from fcntl import ioctl

__all__ = ['Joystick']

JSIOCGVERSION        = 0x00046a01 | ~0x7fffffff
JSIOCGAXES           = 0x00016a11 | ~0x7fffffff
JSIOCGBUTTONS        = 0x00016a12 | ~0x7fffffff
JSIOCGNAME_BASE      = 0x00006a13 | ~0x7fffffff
_IOC_SIZESHIFT       = 0x00000010

_js_event_struct = "=LhBB"
_js_event_size = struct.calcsize(_js_event_struct)

def _find_joystick_path():
    return "/dev/input/js0"

class Joystick(object):
    def __init__(self, f=None):
        if f is None:
            f = _find_joystick_path()
        if isinstance(f, int):
            self.fd = f
        elif isinstance(f, str):
            self.fd = os.open(f, os.O_RDWR)
        else:
            self.fd = os.dup(f.fileno())

    def fileno(self): return self.fd

    def close(self):
        if self.fd != None:
            os.close(self.fd)
        self.fd = None

    def __del__(self):
        self.close()

    def get_name(self, sz=256):
        buf = '\0' * sz
        request = JSIOCGNAME_BASE+ (sz << _IOC_SIZESHIFT)
        return ioctl(self.fd, request, buf).rstrip('\0')
    name = property(get_name)

    def get_version(self):
        return struct.unpack('l', ioctl(self.fd, JSIOCGVERSION, '\0'*4))
    version = property(get_version)

    def get_axes(self):
        return ord(ioctl(self.fd, JSIOCGAXES, '\0'))
    numaxes = property(get_axes)

    def get_buttons(self):
        return ord(ioctl(self.fd, JSIOCGBUTTONS, '\0'))
    numbuttons = property(get_buttons)

    def readable(self):
        r, w, x = select.select([self.fd], [], [], 0)
        return bool(r)

class EventfulJoystick(Joystick):
    def get_next_event(self):
        bytes = os.read(self.fd, _js_event_size)
        return struct.unpack(_js_event_struct, bytes)

class StatelyJoystick(EventfulJoystick):
    def __init__(self, fd=None):
        EventfulJoystick.__init__(self, fd)
        self.axes = [0] * self.numaxes
        self.buttons = [0] * self.numbuttons
        while self.readable(): self.get_next_event()

    def update(self):
        self._update()
        while self.readable(): self._update()

    def _update(self):
        time, value, type, number = self.get_next_event()
        if type == 2:
            self.axes[number] = value
        elif type == 1:
            self.buttons[number] = value

def stest(f=None):
    j = StatelyJoystick(f)
    print "Joystick (%s) has %d axes and %d buttons." % (
                j.name, j.numaxes, j.numbuttons)
    print "Driver version is 0x%x" % j.version
    print "Testing ... (interrupt to exit)"
    try:
        while 1:
            for a in j.axes: print "% 3.1f" % (a/32767.),
            print "|",
            for b in j.buttons: print ".*"[bool(b)],
            print
            j.update()
    except KeyboardInterrupt:
        pass

def etest(f=None):
    j = EventfulJoystick(f)
    print "Joystick (%s) has %d axes and %d buttons." % (
                j.name, j.numaxes, j.numbuttons)
    print "Driver version is 0x%x" % j.version
    print "Testing ... (interrupt to exit)"
    try:
        while 1:
            time, value, type, number = j.get_next_event()
            if type == 1:
                print "<Button%s-%d>" % (["release", "press"][value], number)
            elif type == 2:
                print "<AxisChange-%d> %3.1f" % (number, value / 32767.)
    except KeyboardInterrupt:
        pass

if __name__ == '__main__':
    import sys, getopt
    opts, args = getopt.getopt(sys.argv[1:], "es")
    test = etest
    for k, v in opts:
        if k == '-e': test = etest    
        if k == '-s': test = stest    
    test(*args)
